//
//  UMServer.h
//  ulibserver
//
//  Created by Andreas Fink on 19.07.17.
//  Copyright © 2017 Andreas Fink. All rights reserved.
//

#import <ulib/ulib.h>
#import <ulibasn1/ulibasn1.h>
#import "UMServerProviderProtocol.h"
#import "UMServerUserProtocol.h"

@class UMServerConnection;
@class UMServerMessage;

@interface UMServer : UMLayer<UMServerProviderProtocol>
{
    NSMutableArray *_connections;
    UMHost *localHost;
    int _localPort;

    BOOL    _useTcp;
    BOOL    _useSctp;
    BOOL    _useUdp;
    BOOL    _useSccp;
    BOOL    _useTcap;
    BOOL    _useIpv4;
    BOOL    _useIpv6;

    UMSynchronizedDictionary *_listeners;
    UMSynchronizedDictionary *_listenerShouldBeRunning;
    UMSynchronizedDictionary *_listenerIsRunning;
    BOOL _readerShouldBeRunning;
    BOOL _readerIsRunning;
    
    int _receivePollTimeoutMs;
    NSString *advertizeName;
    UMSleeper *_txSleeper;

    UMSynchronizedDictionary *_portHandlers; /* contains id<UMServerUser> objects */
    UMSynchronizedDictionary *_portServiceNames; /* contains NSStrings */
    UMSynchronizedDictionary *_pendingOutgoingTransactions;
    UMSynchronizedDictionary *_routes;
    NSString *_ownAddress;
    int64_t  lastClientPort;
    int64_t _maxHopCount;
}

@property(readwrite,assign,atomic)  BOOL    useTcp;
@property(readwrite,assign,atomic)  BOOL    useSctp;
@property(readwrite,assign,atomic)  BOOL    useUdp;
@property(readwrite,assign,atomic)  BOOL    useSccp;
@property(readwrite,assign,atomic)  BOOL    useTcap;
@property(readwrite,assign,atomic)  BOOL    useIpv4;
@property(readwrite,assign,atomic)  BOOL    useIpv6;
@property(readwrite,strong,atomic)  NSString *ownAddress;
@property(readwrite,assign,atomic)  int64_t maxHopCount;

- (void)listenerThreadForSocketType:(NSString *)type;
- (void)readerThread;

- (void)startListeners;
- (void)stopListeners;

- (void)setConfig:(NSDictionary *)config applicationContext:(id)ac;

- (UMASN1Sequence *)ownCapabilities;
- (UMASN1Sequence *)portMaps;

- (void)handleIncomingMessage:(UMServerMessage *)msg onConnection:(UMServerConnection *)con;
- (void)sendMessage:(UMServerMessage *)msg onConnection:(UMServerConnection *)con;
- (void)forwardMessage:(UMServerMessage *)msg onConnection:(UMServerConnection *)con;

- (void)setPortHandler:(id<UMServerUserProtocol>)executor
               forPort:(int64_t)port
              withName:(NSString *)name;
- (id<UMServerUserProtocol>)getPorthandlerForPort:(int64_t)port;

- (int64_t)newTransactionPort;
- (UMServerConnection *)connectTo:(NSString *)host port:(int)port name:(NSString *)remoteName reconnect:(BOOL)reconnect;

-(void) addRoute:(NSString *)address connection:(UMServerConnection *)sc;
- (UMServerConnection *)connectionForDestination:(NSString *)address;

- (void) removePortHandlerForPort:(int64_t)port;


@end
